# coding: utf-8

import base64
import os
import jinja2
import yaml

from faker import Factory
from helpers import get_soap_client
from helpers import create_organization
from helpers import create_person
from helpers import param_parser
from names import SYSTEMS
from objects import get_id
from time import sleep
from zipfile import ZipFile


def main():

    # CONFIG DATA ################################
    env = jinja2.Environment()
    temp_path = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        'templates/'
    )
    env.loader = jinja2.FileSystemLoader(temp_path)

    opt = param_parser()
    systems_list = []
    conf_path = os.path.join(os.getenv('CONF_PATH'), 'conf.yaml')
    config = yaml.load(open(conf_path))
    DESTINATION_SYSTEM_ID = config['DESTINATION_SYSTEM']['ID']
    URL = config['RECIPIENT']['URL']
    is_debug = config['DEBUG']
    timeout = config['TIMEOUT']

    if opt['system'].lower() == 'all':
        is_all = True
        for sys_name in config['SOURCE_SYSTEM'].iterkeys():
            systems_list.append(dict(
                system=sys_name.lower(),
                system_id=config['SOURCE_SYSTEM'][sys_name]['ID'],
                system_auth_key=config['SOURCE_SYSTEM'][sys_name]['AUTH_KEY']
            ))
    else:
        is_all = False
        system = SYSTEMS[opt['system']]
        systems_list.append(dict(
            system=system.lower(),
            system_id=config['SOURCE_SYSTEM'][system.upper()]['ID'],
            system_auth_key=config['SOURCE_SYSTEM'][system.upper()]['AUTH_KEY']
        ))
    ##############################################

    # GENERATE DATA ##############################
    persons = []
    organizations = []
    org_exist = False
    persons_exists = False
    fake = Factory.create('ru_RU')
    for system in systems_list:
        SESSION_ID = str(get_id()) + str(get_id())
        if org_exist:
            org_for_all_persons.name = org_for_all_persons._get_name(
                system['system'], fake
            )
        else:
            org_for_all_persons = create_organization(
                region=opt['region'],
                city=opt['city'],
                oktmo=opt['oktmo'],
                system=system['system'],
                faker=fake
            )
            org_exist = True
        if persons_exists:
            for person in persons:
                person.document_id, person.document_series, \
                person.document_number = person._get_document(
                    system['system'], is_child=True
                )
                person.basic_edu_subject = person._get_subject(system)
                person.basic_mark = person._get_mark(
                    system, person.basic_score
                )
                person.common_edu_subject = person._get_subject(system)
                person.common_mark = person._get_mark(
                    system, person.common_score
                )
        else:
            for _ in xrange(opt['persons']):
                persons.append(create_person(
                    fake,
                    org_for_all_persons,
                    system['system'],
                    is_all
                ))
            persons_exists = True
        for _ in xrange(opt['organizations']):
            organizations.append(create_organization(
                region=opt['region'],
                city=opt['city'],
                oktmo=opt['oktmo'],
                system=system['system'],
                faker=fake
            ))
        ##############################################

        # CREATE DATA PACKAGE ########################
        TEMP_FILE_NAME = 'package_{}.xml'.format(system['system'])
        PACKAGE_FILE_NAME = 'data{}.xml'.format(system['system'])
        temp = env.get_template(TEMP_FILE_NAME)
        with open(PACKAGE_FILE_NAME, 'w') as output:
            output.write(temp.render(
                persons=persons,
                organizations=organizations,
                source_system_id=system['system_id'],
                auth_key=system['system_auth_key'],
                destination_system_id=DESTINATION_SYSTEM_ID,
                session_id=SESSION_ID
            ).encode('utf-8'))
        ##############################################

        # CREATE DATA ARCHIVE AND ENCODING ###########
        PACKAGE_ARCHIVE_NAME = PACKAGE_FILE_NAME.replace('xml', 'zip')
        with ZipFile(PACKAGE_ARCHIVE_NAME, 'w') as archive:
            archive.write(PACKAGE_FILE_NAME)

        with open(PACKAGE_ARCHIVE_NAME, 'r') as archive:
            encode_archive = base64.b64encode(archive.read())
        ##############################################

        # CREATE QUERY FILE ##########################
        QUERY_FILE_NAME = 'query.xml'
        query_temp = env.get_template(QUERY_FILE_NAME)
        QUERY_DATA_FILE = 'query_{}_data.xml'.format(system['system'])
        with open(QUERY_DATA_FILE, 'w') as output:
            output.write(query_temp.render(
                xml_package_name=PACKAGE_FILE_NAME,
                query_package_base64=encode_archive,
                auth_key=system['system_auth_key'],
                source_system_id=system['system_id'],
                destination_system_id=DESTINATION_SYSTEM_ID,
                session_id=SESSION_ID
            ).encode('utf-8'))
        ##############################################

        # SEND REQUEST ###############################
        if opt['send'].lower() == 'true':
            client = get_soap_client(URL)
            request = open(QUERY_DATA_FILE, 'r').read()
            client.service.DataTransfer(__inject={'msg': request})

        ##############################################

        # DELETE TEMP FILES ##########################
        if not is_debug:
            for f in (PACKAGE_ARCHIVE_NAME, PACKAGE_FILE_NAME):
                os.remove(f)
        ##############################################

        # DELAY  #####################################
        if is_all:
            sleep(timeout)
        ##############################################

if __name__ == '__main__':
    main()
